; install.scm
; $Id: install.scm,v 1.17 2005/10/11 22:37:40 lioka Exp $

;------------------------------------------------------------------------------
; usage sample:
; (define *pkgs* (list "openssh-clients" "openssh-server"))
;
; (define (run-install packages)
;   (let ((error (run-dialog
;		 `("/packages/install" packages ,packages quit-on-destroy #t))))
;    (or (and (pair? error)
;	     (run-dialog `("/packages/error" error ,error)))
;	error)))
;
; (button "New" (on-click (run-install *pkgs*)))
;------------------------------------------------------------------------------

(use-translation _ "alterator-packages")

caption (_ "Applying changes...")
margin 1 spacing 1

(define *packages-progress-total* #f)
(define *packages-progress-value* #f)
(define *packages-progress-error* '())

(define (packages-install-progress sender dest iface member args)
  (if (and
       (string=? iface "org.altlinux")
       (string=? member "alterator"))
      (let ((command (car args))
            (arg (cadr args))
            (rest (or (and (pair? (cddr args)) (caddr args)) "")))
        (cond

         ;; progress begin
         ((and (string=? command "progress")
               (string=? arg "begin"))
	  (packages-install-label-id text (_ "Preparing...")))

         ;; start of download/install/whatever
         ((and (string=? command "progress")
               (or (string=? arg "download")
                   (string=? arg "install")))
          (set! *packages-progress-total* (string->number rest))
          (packages-install-label-id
           text (or (and (string=? arg "install") ( _"Install")) (_ "Download")))
          (packages-install-bar-id total *packages-progress-total*)
          (packages-install-bar-id value 0)
          (packages-install-text-id text ""))

         ;; in progress
         ((string=? command "step")
          (let ((val (+ (string->number rest)
                        (packages-install-bar-id value))))
            (set! *packages-progress-value* val)
            (packages-install-text-id text-append arg)
            (packages-install-bar-id value val)))

	 ;; error messages
	 ((string=? command "error")
	  (set! *packages-progress-error*
		(cons (cons arg rest)
		      *packages-progress-error*)))

         ;; normal progress end
         ((and (string=? command "progress")
               (string=? arg "end"))
	  (end-dialog (or (null? *packages-progress-error*)
			  *packages-progress-error*)))

	 ;; progress end -- nothing to do
         ((and (string=? command "progress")
               (string=? arg "continue"))
	  (end-dialog #t))

	 ;; progress abort -- sources weren't available
         ((and (string=? command "progress")
               (string=? arg "abort"))
	  (end-dialog #f))

         ;; not reached normally
         (else
          (packages-install-text-id text-append (string-append "XXX" command ":" arg "\n")))))

      ;; FIXME not ours dbus message, fake it
      '(/ctrl/look action "ignore")))

(define (packages-install-ensure-sources)
  (let ((cdrom #f))
    (command-for-each
     (lambda (attribute value)
       (let ((val-list (string-split value #\:)))
	 (and (pair? val-list)
	      (string=? (car val-list) "cdrom")
	      (let* ((cd (cadr val-list))
		     (from (+ (string-index cd #\[) 1))
		     (to (string-index cd #\])))
		(set! cdrom (substring cd from to))))))
     (woo-first-command (woo-read "/packages/sources/active")))
    (or (not cdrom)
	(let ((current
	       (woo-get-option
		(woo-first-command (woo-read "/packages/cdrom/ident")) 'label)))
	  (or (string=? cdrom current)
	      (and
	       (run-dialog `("/packages/cdrom" label ,cdrom))
	       (packages-install-ensure-sources)))))))

(define (packages-install-available packages)
  (filter
   (lambda (e)
     (string=? "available"
	       (woo-catch
		(lambda ()
		  (woo-get-option
		   (woo-first-command
		    (woo-read (string-append "/packages/state/" e))) 'state))
		(lambda () ""))))
   packages))
  
(define (packages-install)
  (if (packages-install-ensure-sources)
      (let* ((packages (not (not (global 'packages))))
	     (available
	      (or (and packages
		       (packages-install-available (global 'packages)))
		  '())))
	(map
	 (lambda (e)
	   (woo-write
	    (string-append "/packages/state/" e) 'state 'install))
	 available)
	(if (and packages (null? available))
	    (woo-write "/packages/control" 'control "continue")
	    (woo-write "/packages/control" 'control "commit")))
      (woo-write "/packages/control" 'control "abort"))

  (vbox margin 1 spacing 1
   sizepolicy 'expanding
   (relative-width 65)
   (id 'packages-install-label-id
       (label (_ "Applying changes...")))
   (id 'packages-install-text-id
       (textbox "" readonly #t))
   (id 'packages-install-bar-id
       (progressbar total (or *packages-progress-total* 1)
		    value (or *packages-progress-value* 0)))))

(on-dbus
 (packages-install-progress sender destination interface dmember args))

(on-destroy
 (and (global 'quit-on-destroy)
      (woo-write "/packages/control" 'control "quit")))

(packages-install)
