; packages
; $Id: packages.scm,v 1.74 2005/10/14 13:47:04 lioka Exp $

(use-translation _ "alterator-packages")

(define *packages* (_ "Packages Installation"))

;-------------------------------------------------------------------------------
; convenience utils
;-------------------------------------------------------------------------------
(define *packages-path*
  (let ((path (dirname (cond-car (lookout-convert-uri '(/packages))))))
    (or (and (string=? path ".") (getcwd)) path)))

(define (packages-pixmap fname)
  (string-append *packages-path* "/" fname ".png"))

(define (packages-with-splash thunk)
  (with-splash-message
   (lambda ()
     (splash-message (_ "Please wait..."))
     (thunk))))

;-------------------------------------------------------------------------------
; transaction logging/replaying
;-------------------------------------------------------------------------------
(include-as-is '/packages/log)

;-------------------------------------------------------------------------------
; view methods
;-------------------------------------------------------------------------------
(define *packages-view-alist* '())

(include-as-is '/packages/view/search)
(include-as-is '/packages/view/pending)
(include-as-is '/packages/view/groups)
(include-as-is '/packages/view/profile)

(define *packages-view* (caar *packages-view-alist*))

(define (packages-view->this)
  (assv-ref *packages-view-alist* *packages-view*))

(define (packages-view->label)
  (assv-ref (packages-view->this) 'label))

(define (packages-view->type)
  (assv-ref (packages-view->this) 'type))

(define (packages-view->add major)
  ((assv-ref (packages-view->this) 'add) major))
  
(define (packages-view->major)
  ((assv-ref (packages-view->this) 'major)))

(define (packages-view->minor minor)
  ((assv-ref (packages-view->this) 'minor) minor))

(define (packages-elt-version elt)
  ((assv-ref (packages-view->this) 'elt-version) elt))

(define (packages-elt-summary elt)
  ((assv-ref (packages-view->this) 'elt-summary) elt))

(define (packages-elt-description elt)
  ((assv-ref (packages-view->this) 'elt-description) elt))

(define (packages-elt-state elt)
  (packages-log-add
   ((assv-ref (packages-view->this) 'elt-state) elt)))

;===============================================================================
; UI
;===============================================================================

;-------------------------------------------------------------------------------
; view selector
;-------------------------------------------------------------------------------
(define (packages-view-update)
  (let ((view (packages-view-id text)))
    (for-each
     (lambda (e)
       (and (string=? view (assv-ref (cdr e) 'label))
	    (set! *packages-view* (car e))))
     *packages-view-alist*))
  (packages-search-update)
  (packages-major-update))

(define (packages-view-contents)
  (items
   (map (lambda (e) (assv-ref e 'label))
	(map cdr *packages-view-alist*))))

(define (packages-view-ui)
  (hbox
   (label (_ "View:"))
   (id 'packages-view-id
       (combobox (packages-view-contents)
		 (on-change (packages-view-update))))))

;-------------------------------------------------------------------------------
; search
;-------------------------------------------------------------------------------
(define (packages-search)
  (packages-view->add (packages-search-keyword-id text))
  (packages-major-update))

(define (packages-search-update)
  (packages-search-id
   hidden (not (assv-ref (packages-view->this) 'add))))

(define (packages-search-ui)
  (id 'packages-search-id
      (hbox
       hidden (not (assv-ref (packages-view->this) 'add))
       (label (_ "Keywords:"))
       (id 'packages-search-keyword-id
	   (edit "" (on-return
		     (packages-with-splash packages-search)))))))

;-------------------------------------------------------------------------------
; major
;-------------------------------------------------------------------------------
(define *packages-major* #f)

(define (packages-major-contents)
  (let ((contents (packages-view->major)))
    (set! *packages-major* (or (and (pair? contents)
				    (car contents))
			       contents))
    (items contents)))

(define (packages-major-update)
  (packages-major-id remove 'all)
  (packages-major-id (packages-major-contents))
  (packages-major-id current 0)
  (packages-major-id 'call on-select))

(define (packages-major-ui)
  (id 'packages-major-id
      (listbox
       sizepolicy 'expanding
       (packages-major-contents)
       (on-select (packages-with-splash packages-minor-update))
       current 0)))

;-------------------------------------------------------------------------------
; minor
;-------------------------------------------------------------------------------
(define *packages-minor* #f)

(define (packages-minor-contents)
  (let* ((major (or (packages-major-id text)
		    *packages-major*))
	 (contents (packages-view->minor major)))
    (set! *packages-minor*
          (or (and (pair? contents) (caar contents))
              contents))
    (items
     (map
      (lambda (elt)
	(cons (car elt) (packages-pixmap (cdr elt)))) contents))))

(define (packages-minor-update)
  (packages-minor-id remove 'all)
  (packages-minor-id (packages-minor-contents))
  (packages-minor-id current 0)
  (packages-minor-id 'call on-select))

(define (packages-minor-state)
  (packages-elt-state (packages-minor-id text))
  (packages-minor-state-1))

(define (packages-minor-state-1)
  (and (eqv? (packages-view->type) 'dynamic)
       (let ((idx (packages-major-id current)))
	 (packages-major-id remove 'all)
	 (packages-major-id (packages-major-contents))
	 (packages-major-id current idx)))
  (let ((idx (packages-minor-id current)))
    (packages-minor-id remove 'all)
    (packages-minor-id (packages-minor-contents))
    (packages-minor-id current idx))
  (packages-status-update))

(define (packages-minor-ui)
  (id 'packages-minor-id
      (listbox
       sizepolicy 'expanding
       (packages-minor-contents)
       (on-select (packages-info-update))
       (on-double-click	(packages-with-splash packages-minor-state))
       current 0)))

;-------------------------------------------------------------------------------
; info
;-------------------------------------------------------------------------------
(define *packages-info-mode* #t)

(define (packages-info-height)
  (or (and *packages-info-mode* 16) 4))

(define packages-info-pixmap
  (lambda ()
    (or (and *packages-info-mode*
             (packages-pixmap "toggle-down"))
        (packages-pixmap "toggle-up"))))

(define (packages-info-text)
  (let ((elt (or (packages-minor-id text)
                 *packages-minor*)))
    (or (and *packages-info-mode*
             (packages-elt-description elt))
        (packages-elt-summary elt))))

(define (packages-info-update)
  (packages-info-id text (packages-info-text)))

(define (packages-info-toggle)
  (set! *packages-info-mode* (not *packages-info-mode*))
  (packages-info-toggle-id pixmap (packages-info-pixmap))
  (packages-info-id height (packages-info-height))
  (packages-info-id text (packages-info-text)))

(define (packages-info-toggle-ui)
  (id 'packages-info-toggle-id
      (button "" flat #t sizepolicy '(minimum . fixed) height 2
              pixmap (packages-info-pixmap)
              (on-click (packages-info-toggle)))))

(define (packages-info-ui)
  (id 'packages-info-id
      (textbox (packages-info-text)
               sizepolicy '(expanding . fixed)
               readonly #t)))

;-------------------------------------------------------------------------------
; status
;-------------------------------------------------------------------------------
(define *packages-status-mode* #f)

(define (packages-status-pixmap)
  (or (and *packages-status-mode*
           (packages-pixmap "toggle-up"))
      (packages-pixmap "toggle-down")))

(define (packages-status-toggle)
  (set! *packages-status-mode* (not *packages-status-mode*))
  (packages-status-toggle-id pixmap (packages-status-pixmap))
  (packages-status-id hidden *packages-status-mode*))

(define (packages-status)
  (let ((reply (woo-first-command (woo-read "/packages/control"))))
    (map (lambda (e) (cons e (woo-get-option reply e)))
         '(available installed install upgrade remove keep dsize isize foobaz))))

(define (packages-status-update)
  (let ((values (packages-status)))
    (packages-status-available-value-id text (assoc-ref values 'available))
    (packages-status-installed-value-id text (assoc-ref values 'installed))
    (packages-status-install-value-id text (assoc-ref values 'install))
    (packages-status-upgrade-value-id text (assoc-ref values 'upgrade))
    (packages-status-remove-value-id text (assoc-ref values 'remove))
    (packages-status-keep-value-id text (assoc-ref values 'keep))
    (packages-status-dsize-value-id text (assoc-ref values 'dsize))
    (packages-status-isize-value-id text (assoc-ref values 'isize))

    (packages-status-isize-label-id
     text
     (or (and (string=? (assoc-ref values 'foobaz) "disk")
              (_ "Be freed on disk"))
         (_ "Be used on disk"))))

  (next-button enabled (packages-log-pending-empty?))
  (prev-button enabled (packages-log-pending-empty?))

  (packages-command-update-id enabled (packages-log-pending-empty?))
  (packages-command-reset-id enabled (not (packages-log-pending-empty?)))
  (packages-command-apply-id enabled (not (packages-log-pending-empty?)))
  (packages-command-undo-id enabled (not (packages-log-undo-empty?)))
  (packages-command-redo-id enabled (not (packages-log-redo-empty?))))

(define (packages-status-entry-genid key sub)
  (string->symbol
   (string-append "packages-status-"
                  (symbol->string key) "-"
                  (symbol->string sub) "-id")))

(define (packages-status-entry text pix val-alist key)
  (hbox
   (label "" pixmap (packages-pixmap pix) sizepolicy 'maximum)
   (id (packages-status-entry-genid key 'label)
       (label text align 'left))
   (id (packages-status-entry-genid key 'value)
       (label (assoc-ref val-alist key) align 'right))))

(define (packages-status-toggle-ui)
  (id 'packages-status-toggle-id
      (button "" flat #t sizepolicy '(minimum . fixed) height 2
	      pixmap (packages-status-pixmap)
	      (on-click (packages-status-toggle)))))

(define (packages-status-ui)
  (let ((values (packages-status)))
    (id 'packages-status-id
	(vbox hidden *packages-status-mode*
	      sizepolicy '(minimum . maximum)
	      (packages-status-entry (_ "Available") "available" values 'available)
	      (packages-status-entry (_ "Installed") "installed" values 'installed)
	      (packages-status-entry (_ "Be Installed") "be-installed" values 'install)
	      (packages-status-entry (_ "Be Upgraded") "be-upgraded" values 'upgrade)
	      (packages-status-entry (_ "Be Kept") "be-kept" values 'keep)
	      (packages-status-entry (_ "Be Removed") "be-removed" values 'remove)
	      (packages-status-entry (_ "Need to get") "moreinfo" values 'dsize)
	      (packages-status-entry (_"Be used on disk") "moreinfo" values 'isize)))))

;-------------------------------------------------------------------------------
; command toolbox
;-------------------------------------------------------------------------------
(define (packages-command-update)
  (let ((result (run-dialog `("/packages/update"))))
    (or (and (pair? result)
	     (run-dialog `("/packages/error" error ,result)))
	(and (not result)
	     (packages-command-reset))))
  (packages-log-reset)
  (packages-view-update)
  (packages-status-update))

(define (packages-command-reset)
  (woo-write "/packages/control" 'control "quit")
  (if (eqv? (packages-view->type) 'dynamic)
      (packages-major-update)
      (packages-minor-update))
  (packages-log-reset)
  (packages-status-update))

(define (packages-command-apply)
  (let ((result (run-dialog `("/packages/install"))))
    (or (and (pair? result)
	     (run-dialog `("/packages/error" error ,result)))
	(and (not result)
	     (packages-command-reset))))
  (packages-log-reset)
  (packages-view-update)
  (packages-status-update))

(define (packages-command-undo)
  (packages-log-undo)
  (packages-minor-state-1))

(define (packages-command-redo)
  (packages-log-redo)
  (packages-minor-state-1))

(define (packages-command-quit)
  (and (not (packages-log-pending-empty?))
       (packages-command-apply))
  (woo-write "/packages/control" 'control "quit"))

(define (packages-command-ui)
  (let* ((values (packages-status))
	 (pending
	  (pair? (memq #f (map (lambda (e) (string=? "0" (assoc-ref values e)))
			       '(install upgrade remove))))))
    (hbox
     (id 'packages-command-update-id
         (button (_ "Update") flat #t sizepolicy '(expanding . maximum)
                 enabled (not pending)
                 (on-click (packages-command-update))))
     
     (id 'packages-command-reset-id
         (button (_ "Reset") flat #t sizepolicy '(expanding . maximum)
                 enabled pending
                 (on-click
                  (packages-with-splash packages-command-reset))))
     
     (id 'packages-command-undo-id
         (button (_ "Undo") flat #t sizepolicy '(expanding . maximum)
                 enabled (not (packages-log-undo-empty?))
                 (on-click
                  (packages-with-splash packages-command-undo))))
     
     (id 'packages-command-redo-id
         (button (_ "Redo") flat #t sizepolicy '(expanding . maximum)
                 enabled (not (packages-log-redo-empty?))
                 (on-click
                  (packages-with-splash packages-command-redo))))
     
     (id 'packages-command-apply-id
         (button (_ "Apply") flat #t sizepolicy '(expanding . maximum)
                 enabled pending
                 (on-click (packages-command-apply)))))))
  
;-------------------------------------------------------------------------------
; main
;-------------------------------------------------------------------------------
(define (packages-ui)
  (vbox
   (groupbox
    ""
    (hbox
     (relative-height 80 page)
     (vbox
      sizepolicy '(fixed . expanding)
      (width 27)
      (packages-view-ui)
      (packages-major-ui)
      (packages-status-toggle-ui)
      (packages-status-ui))
     (vbox
      (packages-search-ui)
      (packages-minor-ui)
      (packages-info-toggle-ui)
      (packages-info-ui))))
   (groupbox
    ""
    (packages-command-ui))))

(packages-ui)
(packages-info-toggle)

; (next-button enabled (packages-log-pending-empty?))
(prev-button enabled (packages-log-pending-empty?))

(on-apply
 (packages-command-quit))

(on-acc-leave
 (and (not (packages-log-pending-empty?)) 'locked))

(on-destroy
  (woo-write "/packages/control" 'control "quit"))
