;;; sources
;;; $Id: sources.scm,v 1.8 2005/10/12 13:24:15 lioka Exp $

(use-modules (ice-9 pretty-print))

(use-translation _ "alterator-packages")
(define *packages-sources-title* (_ "Packages Sources"))

;----------------------------------------------------------------------------
;
(define *packages-path*
  (let ((path (dirname (cond-car (lookout-convert-uri '(/packages))))))
    (or (and (string=? path ".") (getcwd)) path)))

(define (packages-uri-mangle uri)
  (string-join (string-split uri #\:) "#" 'infix))

(define (packages-uri-demangle uri)
  (string-join (string-split uri #\#) ":" 'infix))

;----------------------------------------------------------------------------
;
(define *packages-cdrom-mountpoint*
  (caddr (woo-first-command
          (woo-read "/packages/cdrom/mountpoint"))))

(define *packages-vendors*
  (let ((vendors '()))
    (command-for-each
     (lambda (key value)
       (set! vendors (cons (cons (symbol->string key) value) vendors)))
     (woo-first-command (woo-read "/packages/sources/vendors")))
    ;;(pretty-print vendors (current-error-port))
    (cons (cons "" (_ "<not signed>")) vendors)))

(define *packages-sources*
  (let ((contents '()))
    (for-each
     (lambda (state)
       (command-for-each
        (lambda (uri values)
          (let* ((uri (packages-uri-demangle (symbol->string uri)))
                 (elt (or (assoc-ref contents uri) '()))
                 (val-list (string-split values #\|)))
            (for-each
             (lambda (key value)
               (set! elt (assoc-set! elt key value)))
             (list (string->symbol (car val-list)) 'proto 'key)
             (list state (string->symbol (car (string-split uri #\:))) (cadr val-list)))
            (set! contents (assoc-set! contents uri elt))))
        (woo-first-command
         (woo-read
          (string-append "/packages/sources/" (symbol->string state))))))
     '(passive active))
    ;; (pretty-print contents (current-error-port))
    (acons (_ "<New entry>") '((rpm . passive)
                               (rpm-src . passive)
                               (key . "")) contents)))

;----------------------------------------------------------------------------
;
(define (packages-uri-set! uri key value)
  (set! *packages-sources*
        (assoc-set! *packages-sources* uri
                    (assoc-set! (assoc-ref *packages-sources* uri) key value))))

(define (packages-uri-get uri key)
  (assoc-ref (assoc-ref *packages-sources* uri) key))

(define (packages-uri-key->sig key)
  (or (assoc-ref *packages-vendors* key) ""))

(define (packages-uri-key->nth key)
  (list-index ;; DANGER srfi-1
   (lambda (e) (equal? key e))
   (map car *packages-vendors*)))

(define (packages-uri-sig->key sig)
  (let ((v (filter
            (lambda (e)
              (equal? sig (cdr e)))
            *packages-vendors*)))
    ;; (pretty-print v (current-error-port))
    (or (and (null? v) "") (caar v))))

(define (packages-uri-loc-nth loc)
  (list-index ;; DANGER srfi-1
   (lambda (e) (equal? loc e))
   (map car *packages-sources*)))

(define (packages-uri-pixmap uri)
  (string-append
   *packages-path* "/"
   (or
    (and (eq? (packages-uri-get uri 'rpm) 'passive)
         (eq? (packages-uri-get uri 'rpm-src) 'passive)
         "passive")
    "active")
   ".png"))

(define (packages-sources-items)
  (items
   (map
    (lambda (uri)
      (cons uri (packages-uri-pixmap uri)))
    (map car *packages-sources*))))

;----------------------------------------------------------------------------
; event handlers

(define (packages-uri-update)
  (let* ((uri (packages-sources text))
         (elt (assoc-ref *packages-sources* uri))
         (old-entry (packages-uri-get uri 'proto)))
    
    (packages-uri-cont enabled old-entry)
    (packages-uri-bin checked (and elt (eq? (assoc-ref elt 'rpm) 'active)))
    (packages-uri-src checked (and elt (eq? (assoc-ref elt 'rpm-src) 'active)))
    
    (packages-uri-sig enabled (and old-entry (not (eq? old-entry 'cdrom))))
    (packages-uri-sig-combo current (or (and elt (packages-uri-key->nth (assoc-ref elt 'key))) 0))

    (packages-uri-loc-edit text (or (and old-entry uri) ""))
    (packages-uri-loc-but text (or (and old-entry (_ "Replace")) (_ "Add")))
    
    (packages-uri-cd enabled (or (not old-entry) (eq? old-entry 'cdrom)))))

(define (packages-uri-toggle w what)
  (let ((uri (packages-sources text))
        (state (w checked))
        (key (packages-uri-sig->key (packages-uri-sig-combo text))))
    ;;(format (current-error-port) "what: ~S: state: ~S key: ~S uri: ~S\n" what state key uri)
    (packages-uri-set! uri what (or (and state 'active) 'passive))
    (packages-sources item-pixmap
                      (packages-uri-pixmap uri)
                      (packages-uri-loc-nth uri))
    (woo-write
     (string-append "/packages/sources/" (or (and state "active") "passive"))
     'source (string-append (symbol->string what) "|" key "|" (packages-uri-mangle uri)))))

(define (packages-uri-signature)
  (let* ((uri (packages-sources text))
         (mangled-uri (packages-uri-mangle uri))
         (oldkey (packages-uri-get uri 'key))
         (newkey (packages-uri-sig->key (packages-uri-sig-combo text))))
    ;; (format (current-error-port) "old key: ~S new key: ~S\n" oldkey newkey)
    (or (equal? oldkey newkey)
        (begin
          (packages-uri-set! uri 'key newkey)
          (for-each
           (lambda (w what)
             ;; (format (current-error-port) "do ~S with ~S\n" w what)
             (and (w checked)
                  (for-each
                   (lambda (dest key)
                     (woo-write (string-append "/packages/sources/" dest) 'source
                                (string-append what "|" key "|" mangled-uri)))
                   '("passive" "active") `(,oldkey ,newkey))))
           `(,packages-uri-bin ,packages-uri-src) '("rpm" "rpm-src"))))))

(define (packages-uri-location)
  (let ((olduri (packages-sources text))
        (newuri (packages-uri-loc-edit text)))
    ;;(format (current-error-port) "old uri: ~S\nnew uri: ~S\n" olduri newuri)
    ;; TODO validate newuri
    (if (packages-uri-get olduri 'proto)
        (let ((key (packages-uri-get olduri 'key)))
          (for-each
           (lambda (w what)
             ;; (format (current-error-port) "do ~S with ~S\n" w what)
             (and (w checked)
                  (for-each
                   (lambda (dest uri)
                     (woo-write (string-append "/packages/sources/" dest) 'source
                                (string-append what "|" key "|" (packages-uri-mangle uri))))
                   '("passive" "active") `(,olduri ,newuri))))
           `(,packages-uri-bin ,packages-uri-src) '("rpm" "rpm-src"))
          (let ((nth (packages-uri-loc-nth olduri)))
            (packages-sources item-text newuri nth)
            (list-set! *packages-sources* nth
                       (cons newuri (assoc-ref *packages-sources* olduri)))))
        (let* ((proto (string->symbol (car (string-split newuri #\:))))
               (type (and (eq? proto 'cdrom)
                          (woo-get-option
                           (woo-first-command
                            (woo-read "/packages/cdrom/add")) 'type)))
               (rpm-active (or (and (equal? type "rpm") 'active) 'passive))
               (rpm-src-active (or (and (equal? type "rpm-src") 'active) 'passive)))
          (set! *packages-sources*
                (cons (car *packages-sources*)
                      (acons newuri `((rpm . ,rpm-active)
                                      (rpm-src . ,rpm-src-active)
                                      (key . "")
                                      (proto . ,proto))
                             (cdr *packages-sources*))))
          ;;(pretty-print *packages-sources* (current-error-port))
          (packages-sources remove 'all)
          (packages-sources (packages-sources-items))
          (packages-sources current 1)))))

(define (packages-uri-cdrom)
  (let ((uri (packages-uri-cdrom-ensure)))
    (and uri
         (packages-uri-loc-edit text uri))))

(define (packages-uri-cdrom-ensure)
  (or (let ((uri (woo-get-option
                  (woo-first-command
                   (woo-read "/packages/cdrom/info")) 'uri)))
        (and (not (equal? "" uri))
             (packages-uri-demangle uri)))
      (and (run-dialog '("/packages/cdrom"))
           (packages-uri-cdrom-ensure))))

(define (packages-uri-mountpoint)
  (let ((mountpoint (packages-uri-cd-mnt text)))
    (set! *packages-cdrom-mountpoint* mountpoint)
    (woo-write "/packages/cdrom/mountpoint" 'mountpoint mountpoint)))

;----------------------------------------------------------------------------
; UI

(groupbox
 (_ "Packages Sources")
 (id 'packages-sources
     (listbox (packages-sources-items)
              (on-select (packages-uri-update))
              current 0)))
(groupbox
 (_ "Properties")
 (vbox
  (hbox
   spacing 1
   (id 'packages-uri-cont
       (groupbox
        (_ "Contains")
        (hbox
         (id 'packages-uri-bin
             (checkbox (_ "binary packages") checked #f
                       (on-click (packages-uri-toggle packages-uri-bin 'rpm))))
         (id 'packages-uri-src
             (checkbox (_ "source packages") checked #f
                       (on-click (packages-uri-toggle packages-uri-src 'rpm-src)))))))
   
   (id 'packages-uri-sig
       (groupbox
        (_ "Signed by")
        (id 'packages-uri-sig-combo
            (combobox (items (map cdr *packages-vendors*))
                      (on-change (packages-uri-signature)))))))
  (groupbox
   (_ "URI")
   (hbox
    spacing 1
    (id 'packages-uri-loc-edit
        (edit "" (on-return (packages-uri-location))))
    (id 'packages-uri-loc-but
        (button "" flat #t (on-click (packages-uri-location))))))
  
  (id 'packages-uri-cd
      (groupbox
       "CD/DVD"
       (hbox
        spacing 1
        (label (_ "Mountpoint:"))
        (id 'packages-uri-cd-mnt
            (edit *packages-cdrom-mountpoint*
                  (on-return (packages-uri-mountpoint))))
        (id 'packages-uri-cd-ident
            (button (_ "Identify") flat #t (on-click (packages-uri-cdrom)))))))))
