; $Id: view-pending.scm,v 1.7 2005/07/12 19:03:10 lioka Exp $

(define *packages-pending-major-current* #f)
(define *packages-pending-minor-current* #f)

(define *packages-pending-major-labels*
  (list (cons "keep" (_ "Be Kept"))
	(cons "install" (_ "Be Installed"))
	(cons "remove" (_ "Be Removed"))
	(cons "upgrade" (_ "Be Upgraded"))
	(cons "essential" (_ "Important"))))

(define *packages-pending-major-labels-reverse*
  (map (lambda (e) (cons (cdr e) (car e)))
       *packages-pending-major-labels*))

(define *packages-pending-major-states*
  (list (cons "keep" "upgradable")
	(cons "install" "be-installed")
	(cons "remove" "be-removed")
	(cons "essential" "be-removed")
	(cons "upgrade" "be-upgraded")))

(define (packages-pending-major->state major)
  (assoc-ref *packages-pending-major-states* major))

(define (packages-pending-major->label major)
  (assoc-ref *packages-pending-major-labels* major))

(define (packages-pending-label->major label)
  (assoc-ref *packages-pending-major-labels-reverse* label))

(define (packages-pending-major)
  (let ((names (woo-read-names "/packages/pending"
			       (woo-list "/packages/pending"))))
    (or (and (null? names) names)
	(map packages-pending-major->label
	     (sort names string<)))))

(define (packages-pending-minor label)
  (set! *packages-pending-major-current*
	(packages-pending-label->major label))
  (set! *packages-pending-minor-current*
	(packages-pending-minor-1
	 *packages-pending-major-current*))
  *packages-pending-minor-current*)

(define (packages-pending-minor-1 major)
  (or
   (and major
	(map
	 (lambda (e)
	   (cons e (packages-pending-major->state major)))
	 (let ((address (string-append "/packages/pending/" major)))
	   (sort (woo-read-names address (woo-list address)) string<))))
   '()))

(define (packages-pending-elt elt)
  (woo-first-command
   (woo-read
    (string-append "/packages/pending/"
		   *packages-pending-major-current*  "/" elt))))

(define (packages-pending-elt-version elt)
  (or (and (string=? "" elt) "")
      (woo-get-option (packages-pending-elt elt) 'version)))

(define (packages-pending-elt-summary elt)
  (or (and (string=? "" elt) "")
      (woo-get-option (packages-pending-elt elt) 'summary)))

(define (packages-pending-elt-description elt)
  (or (and (string=? "" elt) "")
      (woo-get-option (packages-pending-elt elt) 'description)))

(define (packages-pending-elt-state elt)
  (let* ((state (assoc-ref *packages-pending-minor-current* elt))
	 (command (cond ((or (string=? state "locked")
			     (string=? state "available")
			     (string=? state "upgradable")
			     (string=? state "be-removed")) "install")
			((or (string=? state "installed")
			     (string=? state "be-installed")
			     (string=? state "be-upgraded")) "remove"))))
    (woo-write
     (string-append "/packages/state/" elt) 'state command)
    (list command elt)))

;------------------------------------------------------------------------------
(set! *packages-view-alist*
      (assoc-set!
       *packages-view-alist*
       'pending
       (list (cons 'label (_ "Pending"))
	     (cons 'type 'dynamic)
	     (cons 'major packages-pending-major)
	     (cons 'minor packages-pending-minor)
	     (cons 'elt-version packages-pending-elt-version)
	     (cons 'elt-summary packages-pending-elt-summary)
	     (cons 'elt-description packages-pending-elt-description)
	     (cons 'elt-state packages-pending-elt-state))))
