; $Id: view-profile.scm,v 1.17 2005/08/08 14:52:36 lioka Exp $

(define *packages-profiles-major-current* #f)
(define *packages-profiles-minor-current* #f)

(define *packages-profile-file*
  "/var/cache/alterator/packages-profile.scm")

(define *profiles*
  (or (and (access? *packages-profile-file* R_OK)
	   (with-input-from-file *packages-profile-file* read))
      '()))

(define (packages-profiles-major)
  (map car *profiles*))

(define (packages-profiles-minor profile)
  (set! *packages-profiles-major-current* profile)
  (set! *packages-profiles-minor-current*
	(packages-profiles-minor-1 profile))
  *packages-profiles-minor-current*)

(define (packages-profiles-minor-1 profile)
  (let ((profile (or (assoc-ref *profiles* profile) '())))
    (map
     (lambda (ele)
       (let ((name (car ele))
	     (states (map
		      (lambda (e)
			(woo-get-option (woo-first-command (woo-read (string-append "/packages/state/" e))) 'state))
		      (assoc-ref (cdr ele) 'contents))))
	 (cons name (cond ((member "available" states) "available")
			  ((member "be-installed" states) "be-installed")
			  (else "installed")))))
     profile)))

(define (packages-profiles-elt elt)
  (assoc-ref (assoc-ref *profiles* *packages-profiles-major-current*) elt))

(define (packages-profiles-elt-description elt)
  (assoc-ref (packages-profiles-elt elt) 'description))

(define (packages-profiles-elt-contents elt)
  (assoc-ref (packages-profiles-elt elt) 'contents))

(define (packages-profiles-elt-state elt)
  (let* ((state (assoc-ref *packages-profiles-minor-current* elt))
	 (command (cond ((or (string=? state "available")
			     (string=? state "upgradable")
			     (string=? state "be-removed")) "install")
			((or (string=? state "installed")
			     (string=? state "be-installed")) "remove")))
         (contents (packages-profiles-elt-contents elt))
	 (trans (cons command contents)))
    (or (packages-log-undo-trans trans)
        (map
         (lambda (e)
           (woo-write (string-append "/packages/state/" e) 'state command))
         contents))
    trans))

(define (packages-profiles-elt-noop elt)
  (make-string 0))

;-------------------------------------------------------------------------------
; run install over all groups in profiles, where state eq 'install

(or (null? *profiles*)
    (with-splash-message
     (lambda ()
       (splash-message (_ "Please wait..."))
       (map
	(lambda (l)
	  (map
	   (lambda (p)
	     (woo-write
	      (string-append "/packages/state/" p) 'state 'install)) l)
	  (packages-log-add (cons "install" l)))
	(let ((accum '()))
	  (map
	   (lambda (pr)
	     (map
	      (lambda (g)
		(let* ((group (cdr g))
		       (state (assoc-ref group 'state))
		       (contents (assoc-ref group 'contents)))
		  (and (eq? state 'required)
		       (set! accum (lset-adjoin equal? accum contents)))))
	      pr))
	   (map cdr *profiles*))
	  accum)))))

;-------------------------------------------------------------------------------
; write profile back, unsetting 'state

(or (null? *profiles*)
    (with-output-to-file *packages-profile-file*
      (lambda ()
	(format #t
		"~S"
		(map
		 (lambda (profile)
		   (let ((header (car profile))
			 (payload (cdr profile)))
		     (cons header
			   (map
			    (lambda (group)
			      (let ((header (car group))
				    (payload (cdr group)))
				(cons header
				      (let ((contents (assq-ref payload 'contents))
					    (description (assq-ref payload 'description)))
					(list (cons 'description description)
					      (cons 'contents contents))))))
			    payload))))
		 *profiles*)))))

;-------------------------------------------------------------------------------
(or (null? *profiles*)
    (set! *packages-view-alist*
	  (assoc-set!
	   *packages-view-alist*
	   'profiles
	   (list (cons 'label (_ "Profiles"))
		 (cons 'type 'static)
		 (cons 'major packages-profiles-major)
		 (cons 'minor packages-profiles-minor)
		 (cons 'elt-version packages-profiles-elt-noop)
		 (cons 'elt-summary packages-profiles-elt-noop)
		 (cons 'elt-description packages-profiles-elt-description)
		 (cons 'elt-state packages-profiles-elt-state)))))
